#!/usr/bin/env python3

import os
import subprocess
import re
import shutil
import time
import json
import sys
import shlex

from scripts.common import *


def isCompatible():
	return 1 if has_p1() and installed_version_is_updatable() else 0


def main(com):
	if not isCompatible():
		log_and_raise_incompatible_installed_fw(com)

	VERSION = get_version_str()

	com.log({'type': 'info', 'code': 111, 'message': 'begin update script', 'time': int(time.time())})

	if os.path.isfile("full_armhf_p1.tar.gz"):
		diff_archive = False
	else:
		diff_archive = True

	if diff_archive:
		assert_file_exists("full_armhf_p2.tar.gz")
		assert_file_exists("delta_armhf_p1.tar.gz")

	if diff_archive:
		subprocess.check_output("pigz -d -c full_armhf_p2.tar.gz | tar xf -", shell=True)
		subprocess.check_output("pigz -d -c delta_armhf_p1.tar.gz | tar xf -", shell=True)
	else:
		subprocess.check_output("pigz -d -c full_armhf_p1.tar.gz | tar xf -", shell=True)

	assert_file_exists("spi.tar")
	assert_file_exists("boot.tar")
	assert_file_exists("rfs.tar")
	assert_file_exists("user.tar")

	if diff_archive:
		assert_file_exists("spi.tar.vcdiff")
		assert_file_exists("boot.tar.vcdiff")
		assert_file_exists("rfs.tar.vcdiff")
		assert_file_exists("user.tar.vcdiff")

	com.log({'type': 'info', 'code': 112, 'message': 'prepare update', 'time': int(time.time())})

	with open("/proc/cmdline", "r") as cmdline_file:
		cmdline = cmdline_file.read()

	m = re.search("root=((/dev/mmcblk\d)p(\d))", cmdline)

	rfs_partition = m.group(1)
	rfs_number = int(m.group(3)) - 1
	boot_device = m.group(2)

	if rfs_number == 1:
		next_rfs_number = 2
		next_rfs_partition = boot_device + 'p3'
		next_image = 'B'
	elif rfs_number == 2:
		next_rfs_number = 1
		next_rfs_partition = boot_device + 'p2'
		next_image = 'A'
	else:
		exit(1)

	subprocess.check_output(shlex.split("mkfs.ext4 -F -L rfs" + str(next_rfs_number) + " " + next_rfs_partition))

	os.mkdir("./mnt")
	os.mkdir("./mnt/config")
	os.mkdir("./mnt/boot")
	os.mkdir("./mnt/rfs")
	os.mkdir("./mnt/user")

	subprocess.check_output(shlex.split("mount -t vfat " + boot_device + "p1 ./mnt/boot"))
	subprocess.check_output(shlex.split("mount -t ext4 " + next_rfs_partition + " ./mnt/rfs"))
	subprocess.check_output(shlex.split("mount -t ext4 " + boot_device + "p4 ./mnt/user"))

	os.mkdir("./archives")

	com.log({'type': 'info', 'code': 114, 'message': 'Extract boot', 'time': int(time.time())})

	os.mkdir("./archives/boot")

	if diff_archive:
		subprocess.check_output("xdelta3 -d -c -s boot.tar boot.tar.vcdiff | tar -x -C ./archives/boot --no-same-owner", shell=True)
	else:
		subprocess.check_output(shlex.split("tar xf boot.tar -C ./archives/boot --no-same-owner"))

	shutil.copy("./archives/boot/dtb_A", "./mnt/boot/dtb_" + next_image)

	shutil.copy("./archives/boot/uImage_A", "./mnt/boot/uImage_" + next_image)

	com.log({'type': 'info', 'code': 115, 'message': 'Extract rfs', 'time': int(time.time())})

	if diff_archive:
		subprocess.check_output("xdelta3 -d -c -s rfs.tar rfs.tar.vcdiff | tar -x -C ./mnt/rfs", shell=True)
	else:
		subprocess.check_output(shlex.split("tar -x -f rfs.tar -C ./mnt/rfs"))

	# ~ if VERSION in CV_21_01:
		# ~ com.log({'type': 'info', 'code': 113, 'message': 'Flash spi', 'time': int(time.time())})

		# ~ os.mkdir("./archives/spi")

		# ~ if diff_archive:
			# ~ subprocess.check_output("xdelta3 -d -c -s spi.tar spi.tar.vcdiff | tar x -C ./archives/spi --no-same-owner",
									# ~ shell=True)
		# ~ else:
			# ~ subprocess.check_output(shlex.split("tar xf spi.tar -C ./archives/spi --no-same-owner"))

		# ~ os.path.exists('/dev/mtd0')
		# ~ os.path.exists('/dev/mtd1')

		# ~ subprocess.check_output(shlex.split("flash_erase /dev/mtd0 0 0"))
		# ~ subprocess.check_output(shlex.split("flash_erase /dev/mtd1 0 0"))

		# ~ subprocess.check_output(shlex.split("nandwrite /dev/mtd0 ./archives/spi/MLO/MLO.byteswap"))
		# ~ subprocess.check_output(shlex.split("nandwrite /dev/mtd1 ./archives/spi/u-boot/u-boot.img"))

	# remove ROOT CA from manufacturer_web.pem
	with open("/mnt/config/manufacturer_web.pem", "r") as cert_file:
		certs=cert_file.read()

	new_certs, certs_subs = re.subn("-----BEGIN CERTIFICATE-----\nMIIB6TCCAY6gAwIBAgIRAM8M0JuN/0xPp2H7D2ZeXoQwCgYIKoZIzj0EAwIwQjEL\nMAkGA1UEBhMCREUxFTATBgNVBAoMDFBlcmluZXQgR21iSDEcMBoGA1UEAwwTUGVy\naW5ldCBFQ0MgUm9vdCBDQTAgFw0yMTAxMDYxNTEwMDJaGA8yMDUwMTIzMDE1MTAw\nMlowQjELMAkGA1UEBhMCREUxFTATBgNVBAoMDFBlcmluZXQgR21iSDEcMBoGA1UE\nAwwTUGVyaW5ldCBFQ0MgUm9vdCBDQTBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IA\nBCcJPKfReZ\+HmuDNGQ7ADzyq1LENCC2I3J0VQ0nKq/tIF0LmN5g\+YmoZpGAHapZM\nwhch323/woVEXDLw9SjEhcCjYzBhMA4GA1UdDwEB/wQEAwIBBjAPBgNVHRMBAf8E\nBTADAQH/MB0GA1UdDgQWBBRE4RDagkZrZjup5ck1iCgxWiWGjTAfBgNVHSMEGDAW\ngBRE4RDagkZrZjup5ck1iCgxWiWGjTAKBggqhkjOPQQDAgNJADBGAiEApvjPs0mb\n1Rs6xghNRu1Bd\+IsA6bohUO9swEPUoMahbgCIQCvH8l3G5Ht\+CrnCUPH1TiIkOJI\nQPZIGDRUuzT6uTgllg==\n-----END CERTIFICATE-----\n", "", certs)
	if certs_subs > 0:
		os.mkdir("./temp")
		os.mkdir("./temp/config")

		subprocess.check_output( "cp /mnt/config/* ./temp/config/", shell=True )

		with open("./temp/config/manufacturer_web.pem", "w") as new_cert_file:
			new_cert_file.write(new_certs)

		subprocess.check_output( shlex.split("mksquashfs ./temp/config ./temp/config.sqsh") )

		subprocess.check_output( shlex.split("umount /mnt/config") )
		subprocess.check_output( shlex.split("flash_erase /dev/mtd2 0 0") )
		subprocess.check_output( shlex.split("nandwrite /dev/mtd2 ./temp/config.sqsh") )
		subprocess.check_output( shlex.split("mount /mnt/config") )

	switch_root_partition(com, next_image)

	unmount_temp_mounts()

	reboot_mica(com)


def clean():
	# try to umount
	unmount_temp_mounts()


if __name__ == '__main__':
	pass
