#!/usr/bin/env python3

import socket
import shutil
import time
import datetime
import json
import glob
import importlib.machinery
import sys

class fwCom():

	def __init__( self, address, port ):
		self.sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
		self.sockFile = self.sock.makefile('r')
		self.sock.connect((address, port))
		self.jsonRpcId=0

	def __del__(self):
		self.sockFile.close()
		self.sock.close()

	def send( self, mes ):

		print(mes)

		self.jsonRpcId += 1

		payload = {
				"method": "setStatusMessage",
				"params": [mes],
				"jsonrpc": "2.0",
				"id": self.jsonRpcId,
		}

		self.sock.sendall( bytes( json.dumps(payload) + "\n", "utf-8") )

		received = json.loads( self.sockFile.readline().strip() )
		if received["result"] == 0 and received["id"] == self.jsonRpcId:
			return 0
		else:
			return 1
			
	#TODO: deprecated
	def log( self, mes ):
		return self.send(mes)
			
	def receive( self ):

		self.jsonRpcId += 1

		payload = {
				"method": "getResponseMessage",
				"params": [],
				"jsonrpc": "2.0",
				"id": self.jsonRpcId,
		}

		self.sock.sendall( bytes( json.dumps(payload) + "\n", "utf-8") )

		received = json.loads( self.sockFile.readline().strip() )
		if received["result"] != {} and received["id"] == self.jsonRpcId:
			return received
		else:
			return None
			
	def waitForResponse( self, timeout ):
		response = None
		timer = timeout
		
		while response == None and timer >= 0:
			time.sleep(0.1)
			response = self.receive()
			timer -= 0.1
			
		return response

def getCompatibleUpdateModule():
	
	count = 0
	compatibleUpdateModule = None
	genericUpdateModule = None
	
	for filename in glob.glob( "scripts/update_*.py" ):
		count += 1
		try:
			currentUpdateModule = importlib.machinery.SourceFileLoader('updateScriptModule_' + str(count), filename).load_module()
		except Exception as e:
			# script not supported or error
			print(str(e))
			break
		
		isCompatible = currentUpdateModule.isCompatible()
		
		if (isCompatible == True) or (isCompatible == 1):
			compatibleUpdateModule = currentUpdateModule
			break
		elif isCompatible == 2:
			genericUpdateModule = currentUpdateModule
		
	if compatibleUpdateModule:
		return compatibleUpdateModule
	if genericUpdateModule:
		com.send( {'type': 'request', 'code': 402, 'message': 'No update for your version found. Use generic update ? (Potentially dangerous !)', 'time': datetime.datetime.now().isoformat()} )
		response = com.waitForResponse(120)
		
		if not response:
			com.send( {'type': 'error', 'code': 131, 'message': 'Timeout while waiting for response', 'time': datetime.datetime.now().isoformat()} )
			raise Exception( "Timeout while waiting for response" )
		
		if response['result']['type'] == 'response' and response['result']['code'] == 604:
			raise Exception( "Update abort, user denied permission" )
			
		if not ( response['result']['type'] == 'response' and response['result']['code'] == 603 ):
			raise Exception( "Update abort, no permission received" )
		
		return genericUpdateModule
	if not compatibleUpdateModule:
		com.send( {'type': 'error', 'code': 311, 'message': 'no update script for this system version available', 'time': datetime.datetime.now().isoformat()} )
		raise Exception('update script exception')


if __name__ == '__main__':

	com = fwCom( "::1", 11222 )

	com.send( {'type': 'info', 'code': 110, 'message': 'check for script compatibility', 'time': datetime.datetime.now().isoformat()} )

	try:
		compatibleUpdateModule = None
		if len( sys.argv ) > 1:
			filename = sys.argv[1]
			compatibleUpdateModule = importlib.machinery.SourceFileLoader('updateScriptModule_0', filename).load_module()
		else:
			compatibleUpdateModule = getCompatibleUpdateModule()
		compatibleUpdateModule.main(com)
	except Exception as e:
		com.send( {'type': 'error', 'code': 310, 'message': str(e), 'time': datetime.datetime.now().isoformat()} )
		try:
			compatibleUpdateModule.clean()
		except:
			com.send( {'type': 'error', 'code': 313, 'message': 'update cleanup failed', 'time': datetime.datetime.now().isoformat()} )

